----------------------------------------------------
-- Company: NTNU
-- Engineer: Tom Erik Tysse
-- Design Name: Downscaler
-- Module Name: Scaler
-- Comment: Implemented downscaling module
----------------------------------------------------

--include this library for file handling in VHDL.
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std;
USE ieee.numeric_std.ALL;
--library std;
--use std.textio.all; 


entity scaler is
  Port ( 
        s_axis_tdata    : in std_logic_vector(31 downto 0);
        aclk            : in std_logic;
        aresetn         : in std_logic;
        s_axis_tvalid   : in std_logic;
        s_axis_tready   : out std_logic := '0';
        m_axis_tdata    : out std_logic_vector(31 downto 0) 
                        := (others => '0');
        m_axis_tvalid   : out std_logic := '0';
        m_axis_tready   : in std_logic;
        m_axis_tlast    : out std_logic
);
end scaler;

architecture Behavioral of scaler is
-- Change WIDTH and HEIGHT to fit image
constant WIDTH      : integer := 512; 
constant HEIGHT     : integer := 512;   

subtype pixel is integer range 0 to 255;
type mem_type is array (integer range 0 to ((3*WIDTH)-1)) of pixel;

signal row_0        : mem_type;
signal row_1        : mem_type;
signal end_of_file  : std_logic := '0';
signal col          : integer := 0;
signal row          : integer := 0;
signal en           : integer := 0;

begin
endfile : process (aclk)
begin
if (aclk = '1' and aclk'event) then
    if (row = HEIGHT) then
        end_of_file <= '1';
    end if;
end if;
end process;
 
read_write : process (aclk)
begin
if (aclk = '1' and aclk'event) then
    if (end_of_file = '0') then  
        if (s_axis_tvalid = '1') then
            s_axis_tready <= '1';
            en <= 1;
            if (row mod 2 = 0) then
                row_0(col) <= to_integer(unsigned(s_axis_tdata));
            else
                row_1(col) <= to_integer(unsigned(s_axis_tdata));
            end if;
        else
            en <= 0;
        end if;
        if(row mod 2 = 1) then
            if (col mod 6 > 2) then
              if (col mod 6 = 3) then
                m_axis_tvalid <= '1';
                 m_axis_tdata <= std_logic_vector
                 (to_unsigned((row_0(col-3) + row_0(col) 
                 + row_1(col-3)  + row_1(col)) / 4,32));
              elsif (col mod 6 = 4) then
                 m_axis_tdata <= std_logic_vector
                 (to_unsigned((row_0(col-3) + row_0(col) 
                 + row_1(col-3) + row_1(col)) / 4,32));
              elsif (col mod 6 = 5) then
                 m_axis_tdata <= std_logic_vector
                 (to_unsigned((row_0(col-3) + row_0(col) 
                 + row_1(col-3) + row_1(col)) / 4,32));
                 m_axis_tvalid <= '0';
              end if;
            end if;    
        else
            en <= 1;
        end if;
            if (en = 1) then            
                col <= col + 1;
                if (col = 3*WIDTH-1) then
                    col <= 0;
                    row <= row + 1;
                end if;
            end if;
    else
        null;
    end if;
end if;
end process;
end Behavioral;