proc start_step { step } {
  set stopFile ".stop.rst"
  if {[file isfile .stop.rst]} {
    puts ""
    puts "*** Halting run - EA reset detected ***"
    puts ""
    puts ""
    return -code error
  }
  set beginFile ".$step.begin.rst"
  set platform "$::tcl_platform(platform)"
  set user "$::tcl_platform(user)"
  set pid [pid]
  set host ""
  if { [string equal $platform unix] } {
    if { [info exist ::env(HOSTNAME)] } {
      set host $::env(HOSTNAME)
    }
  } else {
    if { [info exist ::env(COMPUTERNAME)] } {
      set host $::env(COMPUTERNAME)
    }
  }
  set ch [open $beginFile w]
  puts $ch "<?xml version=\"1.0\"?>"
  puts $ch "<ProcessHandle Version=\"1\" Minor=\"0\">"
  puts $ch "    <Process Command=\".planAhead.\" Owner=\"$user\" Host=\"$host\" Pid=\"$pid\">"
  puts $ch "    </Process>"
  puts $ch "</ProcessHandle>"
  close $ch
}

proc end_step { step } {
  set endFile ".$step.end.rst"
  set ch [open $endFile w]
  close $ch
}

proc step_failed { step } {
  set endFile ".$step.error.rst"
  set ch [open $endFile w]
  close $ch
}

set_msg_config -id {HDL 9-1061} -limit 100000
set_msg_config -id {HDL 9-1654} -limit 100000

start_step init_design
set rc [catch {
  create_msg_db init_design.pb
  debug::add_scope template.lib 1
  create_project -in_memory -part xc7z020clg484-1
  set_property board_part em.avnet.com:zed:part0:1.3 [current_project]
  set_property design_mode GateLvl [current_fileset]
  set_property webtalk.parent_dir C:/Users/Tom/Downloads/Final-20170616T215441Z-001/Final/project_2/project_2/project_2.cache/wt [current_project]
  set_property parent.project_path C:/Users/Tom/Downloads/Final-20170616T215441Z-001/Final/project_2/project_2/project_2.xpr [current_project]
  set_property ip_repo_paths C:/Users/Tom/Downloads/Final-20170616T215441Z-001/Final/project_2/test_up_mario/test_up_mario.cache/ip [current_project]
  set_property ip_output_repo C:/Users/Tom/Downloads/Final-20170616T215441Z-001/Final/project_2/test_up_mario/test_up_mario.cache/ip [current_project]
  set_property ip_cache_permissions {read write} [current_project]
  add_files -quiet C:/Users/Tom/Downloads/Final-20170616T215441Z-001/Final/project_2/project_2/project_2.runs/synth_1/upscale_module.dcp
  link_design -top upscale_module -part xc7z020clg484-1
  close_msg_db -file init_design.pb
} RESULT]
if {$rc} {
  step_failed init_design
  return -code error $RESULT
} else {
  end_step init_design
}

start_step opt_design
set rc [catch {
  create_msg_db opt_design.pb
  catch {write_debug_probes -quiet -force debug_nets}
  opt_design 
  write_checkpoint -force upscale_module_opt.dcp
  catch {report_drc -file upscale_module_drc_opted.rpt}
  close_msg_db -file opt_design.pb
} RESULT]
if {$rc} {
  step_failed opt_design
  return -code error $RESULT
} else {
  end_step opt_design
}

start_step place_design
set rc [catch {
  create_msg_db place_design.pb
  catch {write_hwdef -file upscale_module.hwdef}
  place_design 
  write_checkpoint -force upscale_module_placed.dcp
  catch { report_io -file upscale_module_io_placed.rpt }
  catch { report_utilization -file upscale_module_utilization_placed.rpt -pb upscale_module_utilization_placed.pb }
  catch { report_control_sets -verbose -file upscale_module_control_sets_placed.rpt }
  close_msg_db -file place_design.pb
} RESULT]
if {$rc} {
  step_failed place_design
  return -code error $RESULT
} else {
  end_step place_design
}

start_step route_design
set rc [catch {
  create_msg_db route_design.pb
  route_design 
  write_checkpoint -force upscale_module_routed.dcp
  catch { report_drc -file upscale_module_drc_routed.rpt -pb upscale_module_drc_routed.pb }
  catch { report_timing_summary -warn_on_violation -max_paths 10 -file upscale_module_timing_summary_routed.rpt -rpx upscale_module_timing_summary_routed.rpx }
  catch { report_power -file upscale_module_power_routed.rpt -pb upscale_module_power_summary_routed.pb }
  catch { report_route_status -file upscale_module_route_status.rpt -pb upscale_module_route_status.pb }
  catch { report_clock_utilization -file upscale_module_clock_utilization_routed.rpt }
  close_msg_db -file route_design.pb
} RESULT]
if {$rc} {
  step_failed route_design
  return -code error $RESULT
} else {
  end_step route_design
}

